@@ -15,6 +15,8 @@
 """
 import unittest
 
+from zope.interface.tests import OptimizationTestMixin
+
 
 def _makeInterfaces():
     from zope.interface import Interface
@@ -235,13 +237,26 @@ class BaseAdapterRegistryTests(unittest.TestCase):
         registry.unsubscribe([IB1], None, nomatch) #doesn't raise
         self.assertEqual(len(registry._subscribers), 2)
 
+    def _instance_method_notify_target(self):
+        self.fail("Example method, not intended to be called.")
+
+    def test_unsubscribe_instance_method(self):
+        IB0, IB1, IB2, IB3, IB4, IF0, IF1, IR0, IR1 = _makeInterfaces()
+        registry = self._makeOne()
+        self.assertEqual(len(registry._subscribers), 0)
+        registry.subscribe([IB1], None, self._instance_method_notify_target)
+        registry.unsubscribe([IB1], None, self._instance_method_notify_target)
+        self.assertEqual(len(registry._subscribers), 0)
+
 
 class LookupBaseFallbackTests(unittest.TestCase):
 
-    def _getTargetClass(self):
+    def _getFallbackClass(self):
         from zope.interface.adapter import LookupBaseFallback
         return LookupBaseFallback
 
+    _getTargetClass = _getFallbackClass
+
     def _makeOne(self, uc_lookup=None, uc_lookupAll=None,
                  uc_subscriptions=None):
         if uc_lookup is None:
@@ -249,10 +264,10 @@ class LookupBaseFallbackTests(unittest.TestCase):
                 pass
         if uc_lookupAll is None:
             def uc_lookupAll(self, required, provided):
-                pass
+                raise NotImplementedError()
         if uc_subscriptions is None:
             def uc_subscriptions(self, required, provided):
-                pass
+                raise NotImplementedError()
         class Derived(self._getTargetClass()):
             _uncached_lookup = uc_lookup
             _uncached_lookupAll = uc_lookupAll
@@ -260,14 +275,11 @@ class LookupBaseFallbackTests(unittest.TestCase):
         return Derived()
 
     def test_lookup_w_invalid_name(self):
-        _called_with = []
         def _lookup(self, required, provided, name):
-            _called_with.append((required, provided, name))
-            return None
+            self.fail("This should never be called")
         lb = self._makeOne(uc_lookup=_lookup)
         with self.assertRaises(ValueError):
             lb.lookup(('A',), 'B', object())
-        self.assertEqual(_called_with, [])
 
     def test_lookup_miss_no_default(self):
         _called_with = []
@@ -361,14 +373,12 @@ class LookupBaseFallbackTests(unittest.TestCase):
         self.assertEqual(_results, [c])
 
     def test_lookup1_w_invalid_name(self):
-        _called_with = []
         def _lookup(self, required, provided, name):
-            _called_with.append((required, provided, name))
-            return None
+            self.fail("This should never be called")
+
         lb = self._makeOne(uc_lookup=_lookup)
         with self.assertRaises(ValueError):
             lb.lookup1('A', 'B', object())
-        self.assertEqual(_called_with, [])
 
     def test_lookup1_miss_no_default(self):
         _called_with = []
@@ -489,9 +499,23 @@ class LookupBaseFallbackTests(unittest.TestCase):
         req, prv, _default = object(), object(), object()
         lb = self._makeOne(uc_lookup=_lookup)
         adapted = lb.adapter_hook(prv, req, 'C', _default)
-        self.assertTrue(adapted is _adapter)
+        self.assertIs(adapted, _adapter)
         self.assertEqual(_f_called_with, [req])
 
+    def test_adapter_hook_super_unwraps(self):
+        _f_called_with = []
+        def _factory(context):
+            _f_called_with.append(context)
+            return context
+        def _lookup(self, required, provided, name=''):
+            return _factory
+        required = super(LookupBaseFallbackTests, self)
+        provided = object()
+        lb = self._makeOne(uc_lookup=_lookup)
+        adapted = lb.adapter_hook(provided, required)
+        self.assertIs(adapted, self)
+        self.assertEqual(_f_called_with, [self])
+
     def test_queryAdapter(self):
         _f_called_with = []
         _adapter = object()
@@ -553,39 +577,33 @@ class LookupBaseFallbackTests(unittest.TestCase):
         self.assertEqual(_called_with, [(('A',), 'B')])
 
 
-class LookupBaseTests(LookupBaseFallbackTests):
+class LookupBaseTests(LookupBaseFallbackTests,
+                      OptimizationTestMixin):
 
     def _getTargetClass(self):
         from zope.interface.adapter import LookupBase
         return LookupBase
 
-    def test_optimizations(self):
-        from zope.interface.adapter import LookupBaseFallback
-        try:
-            import zope.interface._zope_interface_coptimizations
-        except ImportError:
-            self.assertIs(self._getTargetClass(), LookupBaseFallback)
-        else:
-            self.assertIsNot(self._getTargetClass(), LookupBaseFallback)
-
 
 class VerifyingBaseFallbackTests(unittest.TestCase):
 
-    def _getTargetClass(self):
+    def _getFallbackClass(self):
         from zope.interface.adapter import VerifyingBaseFallback
         return VerifyingBaseFallback
 
+    _getTargetClass = _getFallbackClass
+
     def _makeOne(self, registry, uc_lookup=None, uc_lookupAll=None,
                  uc_subscriptions=None):
         if uc_lookup is None:
             def uc_lookup(self, required, provided, name):
-                pass
+                raise NotImplementedError()
         if uc_lookupAll is None:
             def uc_lookupAll(self, required, provided):
-                pass
+                raise NotImplementedError()
         if uc_subscriptions is None:
             def uc_subscriptions(self, required, provided):
-                pass
+                raise NotImplementedError()
         class Derived(self._getTargetClass()):
             _uncached_lookup = uc_lookup
             _uncached_lookupAll = uc_lookupAll
@@ -648,13 +666,13 @@ class VerifyingBaseFallbackTests(unittest.TestCase):
         self.assertEqual(_results, [c])
 
     def test_adapter_hook(self):
-        a, b, c = [object(), object(), object()]
+        a, b, _c = [object(), object(), object()]
         def _factory1(context):
             return a
         def _factory2(context):
             return b
         def _factory3(context):
-            return c
+            self.fail("This should never be called")
         _factories = [_factory1, _factory2, _factory3]
         def _lookup(self, required, provided, name):
             return _factories.pop(0)
@@ -670,13 +688,13 @@ class VerifyingBaseFallbackTests(unittest.TestCase):
         self.assertTrue(adapted is b)
 
     def test_queryAdapter(self):
-        a, b, c = [object(), object(), object()]
+        a, b, _c = [object(), object(), object()]
         def _factory1(context):
             return a
         def _factory2(context):
             return b
         def _factory3(context):
-            return c
+            self.fail("This should never be called")
         _factories = [_factory1, _factory2, _factory3]
         def _lookup(self, required, provided, name):
             return _factories.pop(0)
@@ -724,21 +742,13 @@ class VerifyingBaseFallbackTests(unittest.TestCase):
         self.assertEqual(found, tuple(_results_2))
 
 
-class VerifyingBaseTests(VerifyingBaseFallbackTests):
+class VerifyingBaseTests(VerifyingBaseFallbackTests,
+                         OptimizationTestMixin):
 
     def _getTargetClass(self):
         from zope.interface.adapter import VerifyingBase
         return VerifyingBase
 
-    def test_optimizations(self):
-        from zope.interface.adapter import VerifyingBaseFallback
-        try:
-            import zope.interface._zope_interface_coptimizations
-        except ImportError:
-            self.assertIs(self._getTargetClass(), VerifyingBaseFallback)
-        else:
-            self.assertIsNot(self._getTargetClass(), VerifyingBaseFallback)
-
 
 class AdapterLookupBaseTests(unittest.TestCase):
 
@@ -1001,6 +1011,24 @@ class AdapterLookupBaseTests(unittest.TestCase):
         result = alb.queryMultiAdapter((foo,), IBar, default=_default)
         self.assertTrue(result is _default)
 
+    def test_queryMultiAdapter_errors_on_attribute_access(self):
+        # Which leads to using the _empty singleton as "requires"
+        # argument. See https://github.com/zopefoundation/zope.interface/issues/162
+        from zope.interface.interface import InterfaceClass
+        IFoo = InterfaceClass('IFoo')
+        registry = self._makeRegistry()
+        alb = self._makeOne(registry)
+        alb.lookup = alb._uncached_lookup
+        class UnexpectedErrorsLikeAcquisition(object):
+
+            def __getattribute__(self, name):
+                raise RuntimeError("Acquisition does this. Ha-ha!")
+
+        result = alb.queryMultiAdapter(
+            (UnexpectedErrorsLikeAcquisition(),),
+            IFoo,
+        )
+
     def test_queryMultiAdaptor_factory_miss(self):
         from zope.interface.declarations import implementer
         from zope.interface.interface import InterfaceClass
@@ -1059,6 +1087,29 @@ class AdapterLookupBaseTests(unittest.TestCase):
         self.assertTrue(result is _expected)
         self.assertEqual(_called_with, [foo])
 
+    def test_queryMultiAdapter_super_unwraps(self):
+        alb = self._makeOne(self._makeRegistry())
+        def lookup(*args):
+            return factory
+        def factory(*args):
+            return args
+        alb.lookup = lookup
+
+        objects = [
+            super(AdapterLookupBaseTests, self),
+            42,
+            "abc",
+            super(AdapterLookupBaseTests, self),
+        ]
+
+        result = alb.queryMultiAdapter(objects, None)
+        self.assertEqual(result, (
+            self,
+            42,
+            "abc",
+            self,
+        ))
+
     def test__uncached_lookupAll_empty_ro(self):
         from zope.interface.interface import InterfaceClass
         IFoo = InterfaceClass('IFoo')
@@ -1400,7 +1451,7 @@ class Test_utils(unittest.TestCase):
 
     def test__normalize_name_unicode(self):
         from zope.interface.adapter import _normalize_name
-        
+
         USTR = u'ustr'
         self.assertEqual(_normalize_name(USTR), USTR)
 
@@ -1411,16 +1462,3 @@ class Test_utils(unittest.TestCase):
 
     # _lookup, _lookupAll, and _subscriptions tested via their callers
     # (AdapterLookupBase.{lookup,lookupAll,subscriptions}).
-
-
-def test_suite():
-    return unittest.TestSuite((
-        unittest.makeSuite(BaseAdapterRegistryTests),
-        unittest.makeSuite(LookupBaseFallbackTests),
-        unittest.makeSuite(LookupBaseTests),
-        unittest.makeSuite(VerifyingBaseFallbackTests),
-        unittest.makeSuite(VerifyingBaseTests),
-        unittest.makeSuite(AdapterLookupBaseTests),
-        unittest.makeSuite(AdapterRegistryTests),
-        unittest.makeSuite(Test_utils),
-        ))
